
// -*- C++ -*-
/*!
 * @file PortBase.h
 * @brief RTC's Port base class
 * @date $Date: 2007/07/20 16:01:45 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortBase.h,v 1.10.2.1 2007/07/20 16:01:45 n-ando Exp $
 *
 */

/*
 * $Log: PortBase.h,v $
 * Revision 1.10.2.1  2007/07/20 16:01:45  n-ando
 * Useless friend class declaration was deleted.
 *
 * Revision 1.10  2007/04/26 15:31:34  n-ando
 * The header include order was modified to define _REENTRANT before
 * including ace/config-lite.h in Linux systems.
 * In ace 5.4.7 or later, _REENTRANT flag should be defined explicitly.
 *
 * Revision 1.9  2007/04/13 15:53:03  n-ando
 * RTC::OK was changed to RTC::RTC_OK.
 *
 * Revision 1.8  2007/02/04 17:00:59  n-ando
 * Object reference variable was added.
 *
 * Revision 1.7  2007/01/04 00:43:35  n-ando
 * Now, notify_connect() and notify_disconnect() behavior can be customized
 * publishInterfaces(), subscribeInterfaces() and unsubscribeInterfaces().
 *
 * Revision 1.6  2006/12/02 18:50:22  n-ando
 * A trivial fix.
 *
 * Revision 1.5  2006/11/27 09:57:09  n-ando
 * addProvider() function was added for registration of provider.
 * addConsumer() function was added for registration of consumer.
 *
 * Revision 1.4  2006/11/06 01:46:47  n-ando
 * #include <assert.h> was added.
 *
 * Revision 1.3  2006/11/06 01:16:39  n-ando
 * Now PortBase doesn't depend on PortProfileHelper.
 * Class refference manual has been updated.
 *
 * Revision 1.2  2006/10/17 19:06:45  n-ando
 * connect(), disconnect() and disconnect_all() was implemented.
 *
 * Revision 1.1  2006/10/17 10:22:24  n-ando
 * The first commitment.
 *
 */

#ifndef PortBase_h
#define PortBase_h

#include <rtm/RTC.h>

#include <string>
#include <vector>
#include <ace/Guard_T.h>
#include <ace/Recursive_Thread_Mutex.h>
#include <rtm/idl/RTCSkel.h>
#include <rtm/CORBA_SeqUtil.h>
#include <rtm/NVUtil.h>

#include <iostream>
namespace RTC
{
  //  class ConsumerBase;

  /*!
   * @if jp
   * @class PortBase
   * @brief Port ̊NX
   *
   * RTC::Port ̊ƂȂNXB
   * RTC::Port ͂ق UML Port ̊TOpĂAقړ̂̂Ƃ݂Ȃ
   * ƂłBRT R|[lg̃RZvgɂẮA
   * Port ̓R|[lgɕtAR|[lg̃R|[lgƑݍp
   * sړ_łAʏ̃C^[tF[XƊ֘AtB
   * R|[lg Port ʂĊOɑ΂C^[tF[X񋟂܂͗v
   * 邱ƂłAPort͂̐ڑǗSB
   * <p>
   * Port ̋ۃNX́Aʏ RT R|[lgCX^Xɓ
   * A񋟁EvC^[tF[Xo^ART R|[lg
   * o^AOANZX\ Port Ƃċ@\邱Ƃz肵ĂB
   * <p>
   * RTC::Port  CORBA C^[tF[XƂĈȉ̃Iy[V񋟂B
   *
   * - get_port_profile()
   * - get_connector_profiles()
   * - get_connector_profile()
   * - connect()
   * - notify_connect()
   * - disconnect()
   * - notify_disconnect()
   * - disconnect_all()
   *
   * ̃NXł́ÃIy[V̎񋟂B
   * <p>
   * ̃Iy[V̂Aget_port_profile(), get_connector_profiles(),
   * get_connector_profile(), connect(), disconnect(), disconnect_all() ́A
   * TuNXɂēɐU镑ύXKvȂ߁AI[o[Ch
   * 邱Ƃ͐ȂB
   * <p>
   * notify_connect(), notify_disconnect() ɂẮATuNX񋟁Ev
   * C^[tF[X̎ނɉāAU镑ύXKv
   * ȂA𒼐ڃI[o[Ch邱Ƃ͐ꂸA
   * q notify_connect(), notify_disconnect() ̍ɂĂqׂʂ
   * ̊֐Ɋ֘A protected ֐I[o[Ch邱Ƃɂ
   * U镑ύX邱ƂB
   *
   * @else
   *
   *
   * @endif
   */  
  class PortBase
    : public virtual POA_RTC::Port,
      public virtual PortableServer::RefCountServantBase
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * PortBase ̃RXgN^ Port  name Ɏ菉s
     * ƓɁAg CORBA Object ƂĊAg PortProfile
     *  port_ref Ɏg̃IuWFNgt@Xi[B
     *
     * @param name Port ̖O
     *
     * @else
     *
     * @brief Constructor
     *
     * The constructor of the ProtBase class is given the name of this Port
     * and initialized. At the same time, the PortBase activates itself
     * as CORBA object and stores its object reference to the PortProfile's 
     * port_ref member.
     *
     * @param name The name of Port 
     *
     * @endif
     */
    PortBase(const char* name = "");


    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~PortBase();


    /*!
     * @if jp
     *
     * @brief [CORBA interface] PortProfile擾
     *
     * PortێPortProfileԂB
     * PortProfile \͈̂ȉ̃o[B
     *
     * - name              [string ^] Port ̖OB
     * - interfaces        [PortInterfaceProfileList ^] Port ێ
     *                     PortInterfaceProfile ̃V[PX
     * - port_ref          [Port Object ^] Port g̃IuWFNgt@X
     * - connector_profile [ConnectorProfileList ^] Port ݕێ
     *                     ConnectorProfile ̃V[PX
     * - owner             [RTObject Object ^]  Port L
     *                     RTObject̃t@X
     * - properties        [NVList ^] ̑̃vpeBB
     *
     * @return  Port  PortProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the PortProfile of the Port
     *
     * This operation returns the PortProfile of the Port.
     * PortProfile struct has the following members,
     *
     * - name              [string ] The name of the Port.
     * - interfaces        [PortInterfaceProfileList ^] The sequence of 
     *                     PortInterfaceProfile owned by the Port
     * - port_ref          [Port Object] The object reference of the Port.
     * - connector_profile [ConnectorProfileList ^] The sequence of 
     *                     ConnectorProfile owned by the Port.
     * - owner             [RTObject Object] The object reference of 
     *                     RTObject that is owner of the Port.
     * - properties        [NVList] The other properties.
     *
     * @return the PortProfile of the Port
     *
     * @endif
     */
    virtual PortProfile* get_port_profile();


    /*!
     * @if jp
     *
     * @brief [CORBA interface] ConnectorProfileList擾
     *
     * Portێ ConnectorProfile  sequence ԂB
     * ConnectorProfile  Port Ԃ̐ڑvt@Cێ\̂łA
     * ڑPortԂŏsA֘A邷ׂĂ Port œ̒l
     * ێB
     * ConnectorProfile ͈ȉ̃o[ێĂB
     *
     * - name         [string ^] ̃RlN^̖OB
     * - connector_id [string ^] j[NID
     * - ports        [Port sequnce] ̃RlN^Ɋ֘A Port ̃IuWFNg
     *                t@X̃V[PXB
     * - properties   [NVList ^] ̑̃vpeBB
     *
     * @return  Port  ConnectorProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the ConnectorProfileList of the Port
     *
     * This operation returns a list of the ConnectorProfiles of the Port.
     * ConnectorProfile includes the connection information that describes 
     * relation between (among) Ports, and Ports exchange the ConnectionProfile
     * on connection process and hold the same information in each Port.
     * ConnectionProfile has the following members,
     *
     * - name         [string] The name of the connection.
     * - connector_id [string] Unique identifier.
     * - ports        [Port sequnce] The sequence of Port's object reference
     *                that are related the connection.
     * - properties   [NVList] The other properties.
     *
     * @return the ConnectorProfileList of the Port
     *
     * @endif
     */
    virtual ConnectorProfileList* get_connector_profiles();


    /*!
     * @if jp
     *
     * @brief [CORBA interface] ConnectorProfile 擾
     *
     * connector_id Ŏw肳ꂽ ConnectorProfile ԂB
     *
     * @param connector_id ConnectorProfile  ID
     * @return connector_id  ConnectorProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the ConnectorProfile
     *
     * This operation returns the ConnectorProfiles specified connector_id.
     *
     * @param connector_id ID of the ConnectorProfile
     * @return the ConnectorProfile identified by the connector_id
     *
     * @endif
     */
    virtual ConnectorProfile* get_connector_profile(const char* connector_id);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑs
     *
     * ^ꂽ ConnectoionProfile ɂāAPortԂ̐ڑmB
     * AvP[VvÓA̃R|[lg
     * Port ڑꍇAK؂ȒlZbg ConnectorProfile 
     * connect() ̈Ƃė^ăR[邱ƂɂA֘A Port 
     * ڑmB
     *
     * connect() ɗ^ ConnectorProfile ̃o[̂Aname, ports, 
     * (properties) o[ɑ΂ăf[^ZbgȂ΂ȂȂB
     *
     * @param connector_profile ConnectorProfile
     * @return ReturnCode_t Iy[Ṽ^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Connect the Port
     *
     * This operation establishes connection according to the given 
     * ConnectionProfile inforamtion. 
     * Application programs, which is going to establish the connection 
     * among Ports owned by RT-Components, have to set valid values to the 
     * ConnectorProfile and give it to the argument of connect() operation.
     * 
     * name, ports, (properties) members of ConnectorProfile should be set
     * valid values before giving to the argument of connect() operation.
     *
     * @param connector_profile The ConnectorProfile.
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t connect(ConnectorProfile& connector_profile);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑʒms
     *
     * ̃Iy[V́APortԂ̐ڑsۂɁAPortԂœI
     * Ă΂Iy[VłB
     *
     * @param connector_profile ConnectorProfile
     * @return ReturnCode_t Iy[Ṽ^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Notify the Ports connection
     *
     * This operation is invoked between Ports internally when the connection
     * is established.
     *
     * @param connector_profile The ConnectorProfile.
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t notify_connect(ConnectorProfile& connector_profile);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑ
     *
     * ̃Iy[V͐ڑmɐڑɑ΂ė^ connector_id 
     * ΉsA Port Ƃ̐ڑB
     *
     * @param connector_id ConnectorProfile  ID
     * @return ReturnCode_t Iy[Ṽ^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Connect the Port
     *
     * This operation destroys connection between this port and the peer port
     * according to given id that is given when the connection established.
     *
     * @param connector_id The ID of the ConnectorProfile.
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t disconnect(const char* connector_id);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑʒms
     *
     * ̃Iy[V́APortԂ̐ڑsۂɁAPortԂœI
     * Ă΂Iy[VłB
     *
     * @param connector_id ConnectorProfile  ID
     * @return ReturnCode_t Iy[Ṽ^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Notify the Ports disconnection
     *
     * This operation is invoked between Ports internally when the connection
     * is destroied.
     *
     * @param connector_id The ID of the ConnectorProfile.
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t notify_disconnect(const char* connector_id);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̑Sڑ
     *
     * ̃Iy[V͂ Port Ɋ֘ASĂ̐ڑB
     *
     * @return ReturnCode_t Iy[Ṽ^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Connect the Port
     *
     * This operation destroys all connection channels owned by the Port.
     *
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t disconnect_all();


    //============================================================
    // Local operations
    //============================================================
    /*!
     * @if jp
     * @brief Port ̖Oݒ肷
     *
     * Port ̖Oݒ肷B̖O Port ێ PortProfile.name
     * ɔfB
     *
     * @param name Port ̖O
     *
     * @else
     * @brief Set the name of this Port
     *
     * This operation sets the name of this Port. The given Port's name is
     * applied to Port's PortProfile.name.
     *
     * @param name The name of this Port.
     *
     * @endif
     */
    void setName(const char* name);


    /*!
     * @if jp
     * @brief PortProfile擾
     *
     * Portێ PortProfile  const QƂԂB
     *
     * @return  Port  PortProfile
     *
     * @else
     * @brief Get the PortProfile of the Port
     *
     * This operation returns const reference of the PortProfile.
     *
     * @return the PortProfile of the Port
     *
     * @endif
     */
    const PortProfile& getProfile() const;
    

    /*!
     * @if jp
     *
     * @brief Port ̃IuWFNgQƂݒ肷
     *
     * ̃Iy[V Port  PortProfile ɂ Port g
     * IuWFNgQƂݒ肷B
     *
     * @param  Port ̃IuWFNgQ
     *
     * @else
     *
     * @brief Set the object reference of this Port
     *
     * This operation sets the object reference itself
     * to the Port's PortProfile.
     *
     * @param The object reference of this Port.
     *
     * @endif
     */
    void setPortRef(Port_ptr port_ref);


    /*!
     * @if jp
     *
     * @brief Port ̃IuWFNgQƂ擾
     *
     * ̃Iy[V Port  PortProfile ێĂ
     *  Port g̃IuWFNgQƂ擾B
     *
     * @return  Port ̃IuWFNgQ
     *
     * @else
     *
     * @brief Get the object reference of this Port
     *
     * This operation returns the object reference
     * that is stored in the Port's PortProfile.
     *
     * @return The object reference of this Port.
     *
     * @endif
     */
    Port_ptr getPortRef();


    /*!
     * @if jp
     *
     * @brief Port  owner  RTObject w肷
     *
     * ̃Iy[V Port  PortProfile.owner ݒ肷B
     *
     * @param owner  Port L RTObject ̎Q
     *
     * @else
     *
     * @brief Set the owner RTObject of the Port
     *
     * This operation sets the owner RTObject of this Port.
     *
     * @param owner The owner RTObject's reference of this Port
     *
     * @endif
     */
    void setOwner(RTObject_ptr owner);


    //============================================================
    // protected operations
    //============================================================
  protected:
    /*! @if jp
     *
     * @brief Interface J
     *
     * ̃Iy[V́Anotify_connect() V[PX̎n߂ɃR[
     * 鏃z֐łB
     * notify_connect() ł́A
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * ̏ protected ֐R[ڑsB
     * <br>
     *  Port ł͂̃Iy[VI[o[ChAƂ
     * ^ꂽ ConnectorProfile ɏ]sAp[^sK
     * ł΁ARteurnCode_t ^̃G[R[hԂB
     * ʏ publishInterafaces() ɂẮA Port ɑ
     * C^[tF[XɊւ ConnectorProfile ɑ΂ēK؂ɐݒ肵
     *  Port ɒʒmȂ΂ȂȂB
     * <br>
     * ܂Å֐R[iKł́A Port  Interface Ɋւ
     * ׂ͂Ċ܂܂ĂȂ̂ŁA Port  Interface 擾鏈
     *  subscribeInterfaces() ōsׂłB
     * <br>
     * ̃Iy[V́AVK connector_id ɑ΂Ă͐ڑ̐A
     *  connector_id ɑ΂Ă͍XVK؂ɍsKvB
     *
     * @param connector_profile ڑɊւvt@C
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Publish interface information
     *
     * This operation is pure virutal method that would be called at the
     * beginning of the notify_connect() process sequence.
     * In the notify_connect(), the following methods would be called in order.
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile() 
     *
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and if the given parameter
     * is invalid, it would return error code of ReturnCode_t.
     * Usually, publishInterfaces() method should set interfaces information
     * owned by this Port, and publish it to the other Ports.
     * <br>
     * When this method is called, other Ports' interfaces information may not
     * be completed. Therefore, the process to obtain other Port's interfaces
     * information should be done in the subscribeInterfaces() method.
     * <br>
     * This operation should create the new connection for the new
     * connector_id, and should update the connection for the existing
     * connection_id.
     *
     * @param connector_profile The connection profile information
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile) = 0;
    

    /*!
     * @if jp
     *
     * @brief  Port ɑ΂ notify_connect() R[
     *
     * ConnectorProfile  port_ref Ɋi[Ă Port ̃IuWFNg
     * t@X̃V[PX̒Ag Port ̎ Port ɑ΂
     * notify_connect() R[B
     *
     * @param connector_profile ڑɊւvt@C
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Call notify_connect() of the next Port
     *
     * This operation calls the notify_connect() of the next Port's 
     * that stored in ConnectorProfile's port_ref sequence.
     *
     * @param connector_profile The connection profile information
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t connectNext(ConnectorProfile& connector_profile);


    /*!
     * @if jp
     *
     * @brief  Port ɑ΂ notify_disconnect() R[
     *
     * ConnectorProfile  port_ref Ɋi[Ă Port ̃IuWFNg
     * t@X̃V[PX̒Ag Port ̎ Port ɑ΂
     * notify_disconnect() R[B
     *
     * @param connector_profile ڑɊւvt@C
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Call notify_disconnect() of the next Port
     *
     * This operation calls the notify_disconnect() of the next Port's 
     * that stored in ConnectorProfile's port_ref sequence.
     *
     * @param connector_profile The connection profile information
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t disconnectNext(ConnectorProfile& connector_profile);


    /*! @if jp
     *
     * @brief Interface 擾
     *
     * ̃Iy[V́Anotify_connect() V[PX̒ԂɃR[
     * 鏃z֐łB
     * notify_connect() ł́A
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * ̏ protected ֐R[ڑsB
     * <br>
     *  Port ł͂̃Iy[VI[o[ChAƂ
     * ^ꂽ ConnectorProfile ɏ]sAp[^sK
     * ł΁ARteurnCode_t ^̃G[R[hԂB
     *  ConnectorProfile ɂ͑ Port  Interface Ɋւ
     * SĊ܂܂ĂB
     * ʏ subscribeInterafaces() ɂẮA Port gp
     * Interface Ɋւ擾AṽC^[tF[Xɑ΂
     * ݒ肵Ȃ΂ȂȂB
     * <br>
     * ̃Iy[V́AVK connector_id ɑ΂Ă͐ڑ̐A
     *  connector_id ɑ΂Ă͍XVK؂ɍsKvB
     *
     * @param connector_profile ڑɊւvt@C
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Publish interface information
     *
     * This operation is pure virutal method that would be called at the
     * mid-flow of the notify_connect() process sequence.
     * In the notify_connect(), the following methods would be called in order.
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and if the given parameter
     * is invalid, it would return error code of ReturnCode_t.
     * The given argument ConnectorProfile includes all the interfaces
     * information in it.
     * Usually, subscribeInterafaces() method obtains information of interfaces
     * from ConnectorProfile, and should set it to the interfaces that require
     * them.
     * <br>
     * This operation should create the new connection for the new
     * connector_id, and should update the connection for the existing
     * connection_id.
     *
     * @param connector_profile The connection profile information
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile) = 0;
    
    
    /*! @if jp
     *
     * @brief Interface ̐ڑ
     *
     * ̃Iy[V́Anotify_disconnect() V[PX̏IɃR[
     * 鏃z֐łB
     * notify_disconnect() ł́A
     * - disconnectNext()
     * - unsubscribeInterfaces()
     * - eraseConnectorProfile()
     * ̏ protected ֐R[ڑsB
     * <br>
     *  Port ł͂̃Iy[VI[o[ChAƂ
     * ^ꂽ ConnectorProfile ɏ]ڑsB
     *
     * @param connector_profile ڑɊւvt@C
     *
     * @else
     *
     * @brief Disconnect interface connection
     *
     * This operation is pure virutal method that would be called at the
     * end of the notify_disconnect() process sequence.
     * In the notify_disconnect(), the following methods would be called.
     * - disconnectNext()
     * - unsubscribeInterfaces()
     * - eraseConnectorProfile() 
     *  <br>
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and disconnect interface
     * connection.
     *
     * @param connector_profile The connection profile information
     *
     * @endif
     */
    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile) = 0;


    //============================================================
    // protected utility functions
    //============================================================
    /*!
     * @if jp
     *
     * @brief ConnectorProfile  connector_id tB[h󂩂ǂ
     *
     * @return ŗ^ꂽ ConnectorProfile  connector_id ł΁A
     *         trueAłȂ false ԂB
     *
     * @else
     *
     * @brief Whether connector_id of ConnectorProfile is empty
     *
     * @return If the given ConnectorProfile's connector_id is empty string,
     *         it returns true.
     *
     * @endif
     */
    bool isEmptyId(const ConnectorProfile& connector_profile) const;


    /*!
     * @if jp
     *
     * @brief UUID𐶐
     *
     * ̃Iy[V UUID 𐶐B
     *
     * @return uuid
     *
     * @else
     *
     * @brief Get the UUID
     *
     * This operation generates UUID.
     *
     * @return uuid
     *
     * @endif
     */
    const std::string getUUID() const;


    /*!
     * @if jp
     *
     * @brief UUID𐶐 ConnectorProfile ɃZbg
     *
     * ̃Iy[V UUID 𐶐AConnectorProfile ɃZbgB
     *
     * @param connector_profile connector_id Zbg ConnectorProfile
     *
     * @else
     *
     * @brief Create and set the UUID to the ConnectorProfile
     *
     * This operation generates and set UUID to the ConnectorProfile.
     *
     * @param connector_profile ConnectorProfile to be set connector_id
     *
     * @endif
     */
    void setUUID(ConnectorProfile& connector_profile) const;


    /*!
     * @if jp
     *
     * @brief id  ConnectorProfile ̂̂ǂ肷
     *
     * ̃Iy[V͗^ꂽ ID  ConnectorProfile ̃Xg
     * ݂邩ǂ肷B
     *
     * @param id 肷 connector_id
     *
     * @else
     *
     * @brief Whether the given id exists in stored ConnectorProfiles
     *
     * This operation returns boolean whether the given id exists in 
     * the Port's ConnectorProfiles.
     *
     * @param id connector_id to be find in Port's ConnectorProfiles
     *
     * @endif
     */
    bool isExistingConnId(const char* id);


    /*!
     * @if jp
     *
     * @brief id  ConnectorProfile T
     *
     * ̃Iy[V͗^ꂽ ID  ConnectorProfile  Port 
     *  ConnectorProfile ̃XgTB
     * A id  ConnectorProfile Ȃ΁A ConnectorProfile
     * ԂB
     *
     * @param id  connector_id
     * @return connector_id  ConnectorProfile
     *
     * @else
     *
     * @brief Find ConnectorProfile with id
     *
     * This operation returns ConnectorProfile with the given id from Port's
     * ConnectorProfiles' list.
     * If the ConnectorProfile with connector id that is identical with the
     * given id does not exist, empty ConnectorProfile is returned.
     *
     * @param id the connector_id to be searched in Port's ConnectorProfiles
     * @return CoonectorProfile with connector_id
     *
     * @endif
     */
    ConnectorProfile findConnProfile(const char* id);
    

    /*!
     * @if jp
     *
     * @brief id  ConnectorProfile T
     *
     * ̃Iy[V͗^ꂽ ID  ConnectorProfile  Port 
     *  ConnectorProfile ̃XgTCfbNXԂB
     * A id  ConnectorProfile Ȃ΁A-1 ԂB
     *
     * @param id  connector_id
     * @return Port  ConnectorProfile Xg̃CfbNX
     *
     * @else
     *
     * @brief Find ConnectorProfile with id
     *
     * This operation returns ConnectorProfile with the given id from Port's
     * ConnectorProfiles' list.
     * If the ConnectorProfile with connector id that is identical with the
     * given id does not exist, empty ConnectorProfile is returned.
     *
     * @param id the connector_id to be searched in Port's ConnectorProfiles
     * @return The index of ConnectorProfile of the Port
     *
     * @endif
     */
    CORBA::Long findConnProfileIndex(const char* id);


    /*!
     * @if jp
     *
     * @brief ConnectorProfile ̒ǉ͍XV
     *
     * ̃Iy[V͗^ꂽ^ꂽ ConnectorProfile 
     * Port ɒǉ͍XVۑB
     * ^ꂽ ConnectorProfile  connector_id Ɠ ID 
     * ConnectorProfile XgɂȂ΁AXgɒǉA
     *  ID ݂ ConnectorProfile ㏑ۑB
     *
     * @param coonector_profile ǉ͍XV ConnectorProfile
     *
     * @else
     *
     * @brief Append or update the ConnectorProfile list
     *
     * This operation appends or updates ConnectorProfile of the Port
     * by the given ConnectorProfile.
     * If the connector_id of the given ConnectorProfile does not exist
     * in the Port's ConnectorProfile list, the given ConnectorProfile would be
     * append to the list. If the same id exists, the list would be updated.
     *
     * @param connector_profile the ConnectorProfile to be appended or updated
     *
     * @endif
     */
    void updateConnectorProfile(const ConnectorProfile& connector_profile);


    /*!
     * @if jp
     *
     * @brief ConnectorProfile 폜
     *
     * ̃Iy[V Port  PortProfile ێĂ
     * ConnectorProfileList ̂^ꂽ id  ConnectorProfile
     * 폜B
     *
     * @param id 폜 ConnectorProfile  id
     *
     * @else
     *
     * @brief Delete the ConnectorProfile
     *
     * This operation deletes a ConnectorProfile specified by id from
     * ConnectorProfileList owned by PortProfile of this Port.
     *
     * @param id The id of the ConnectorProfile to be deleted.
     *
     * @endif
     */
    bool eraseConnectorProfile(const char* id);


    /*!
     * @if jp
     *
     * @brief PortInterfaceProfile  C^[tF[Xo^
     *
     * ̃Iy[V Port  PortProfile ́APortInterfaceProfile
     * ɃC^[tF[X̏ǉB
     * ̏́Aget_port_profile() ē PortProfile ̂
     * PortInterfaceProfile ̒lύX݂̂łAۂɃC^[tF[X
     * 񋟂v肷ꍇɂ́ATuNXŁApublishInterface(),
     * subscribeInterface() ̊֐K؂ɃI[o[ChC^[tF[X
     * 񋟁AvsȂ΂ȂȂB
     *
     * C^[tF[X(̃CX^X) Port ňӂłȂ΂ȂȂB
     * ̃C^[tF[Xłɓo^ĂꍇÅ֐ false 
     * ԂB
     *
     * @param name C^[tF[X̃CX^X̖O
     * @param type_name C^[tF[X̌^̖O
     * @param pol C^[tF[X̑ (RTC::PROVIDED  RTC:REQUIRED)
     * @return ̃C^[tF[Xɓo^Ă false ԂB
     *
     * @else
     *
     * @brief Append an interface to the PortInterfaceProfile
     *
     * This operation appends interface information to the PortInterfaceProfile
     * that is owned by the Port.
     * The given interfaces information only updates PortInterfaceProfile of
     * PortProfile that is obtained through get_port_profile().
     * In order to provide and require interfaces, proper functions (for
     * example publishInterface(), subscribeInterface() and so on) should be
     * overridden in subclasses, and these functions provide concrete interface
     * connection and disconnection functionality.
     *
     * The interface (instance) name have to be unique in the Port.
     * If the given interface name is identical with stored interface name,
     * this function returns false.
     *
     * @param name The instance name of the interface.
     * @param type_name The type name of the interface.
     * @param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
     * @return false would be returned if the same name is already registered.
     *
     * @endif
     */
    bool appendInterface(const char* name, const char* type_name,
			 PortInterfacePolarity pol);


    /*!
     * @if jp
     *
     * @brief PortInterfaceProfile C^[tF[Xo^폜
     *
     * ̃Iy[V Port  PortProfile ́APortInterfaceProfile
     * C^[tF[X̏폜B
     *
     * @param name C^[tF[X̃CX^X̖O
     * @param pol C^[tF[X̑ (RTC::PROVIDED  RTC:REQUIRED)
     * @return C^[tF[Xo^ĂȂ false ԂB
     *
     * @else
     *
     * @brief Delete an interface from the PortInterfaceProfile
     *
     * This operation deletes interface information from the
     * PortInterfaceProfile that is owned by the Port.
     *
     * @param name The instance name of the interface.
     * @param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
     * @return false would be returned if the given name is not registered.
     *
     * @endif
     */
    bool deleteInterface(const char* name, PortInterfacePolarity pol);
    

    /*!
     * @if jp
     *
     * @brief PortProfile  properties  NameValue lǉ
     *
     * @param ValueType properties  value ɒǉľ^
     * @param key properties  name
     * @param value properties  value
     *
     * @else
     *
     * @brief Add NameValue data to PortProfile's properties
     *
     * @param ValueType The type of value
     * @param key The name of properties
     * @param value The value of properties
     *
     * @endif
     */
    template <class ValueType>
    void addProperty(const char* key, ValueType value)
    {
      CORBA_SeqUtil::push_back(m_profile.properties,
			       NVUtil::newNV(key, value));
    }


  protected:
    /*!
     * @if jp
     * @brief Port  PortProfile
     * @else
     * @brief PortProfile of the Port
     * @endif
     */
    PortProfile m_profile;
    RTC::Port_var m_objref;
    mutable ACE_Recursive_Thread_Mutex m_profile_mutex;
    typedef ACE_Guard<ACE_Recursive_Thread_Mutex> Guard;
    
    //============================================================
    // Functor
    //============================================================
    /*!
     * @if jp
     * @brief instance_name  PortInterfaceProfile T Functor
     * @else
     * @brief A functor to find a PortInterfaceProfile named instance_name
     * @endif
     */
    struct if_name
    {
      if_name(const char* name) : m_name(name) {};
      bool operator()(const PortInterfaceProfile& prof)
      {
	return m_name == std::string(prof.instance_name);
      }
      std::string m_name;
    };
    
    
    /*!
     * @if jp
     * @brief id  ConnectorProfile T Functor
     * @else
     * @brief A functor to find a ConnectorProfile named id
     * @endif
     */
    struct find_conn_id
    {
      find_conn_id(const char* id) : m_id(id) {};
      bool operator()(const ConnectorProfile& cprof)
      {
	return m_id == std::string(cprof.connector_id);
      }
      std::string m_id;
    };


    /*!
     * @if jp
     * @brief RXgN^ port_ref ƓIuWFNgQƂT Functor
     * @else
     * @brief A functor to find the object reference that is identical port_ref
     * @endif
     */
    struct find_port_ref
    {
      find_port_ref(Port_ptr port_ref) : m_port(port_ref) {};
      bool operator()(Port_ptr port_ref)
      {
	return m_port->_is_equivalent(port_ref);
      }
      Port_ptr m_port;
    };
	
    
    /*!
     * @if jp
     * @brief Port ̐ڑs Functor
     * @else
     * @brief A functor to connect Ports
     * @endif
     */
    struct connect_func
    {
      Port_var port_ref;
      ConnectorProfile connector_profile;
      ReturnCode_t return_code;
      
      connect_func() {};
      connect_func(Port_ptr p, ConnectorProfile& prof)
	: port_ref(p), connector_profile(prof) {};
      void operator()(Port_ptr p)
      {
	if (!port_ref->_is_equivalent(p))
	  {
	    ReturnCode_t retval;
	    retval = p->notify_connect(connector_profile);
	    if (retval != RTC::RTC_OK)
	      {
		return_code = retval;
	      }
	  }
      }
    };
    
    
    /*!
     * @if jp
     * @brief Port ̐ڑs Functor
     * @else
     * @brief A functor to disconnect Ports
     * @endif
     */
    struct disconnect_func
    {
      Port_var port_ref;
      ConnectorProfile connector_profile;
      ReturnCode_t return_code;
      
      disconnect_func() : return_code(RTC::RTC_OK) {};
      disconnect_func(Port_ptr p, ConnectorProfile& prof)
	: port_ref(p), connector_profile(prof), return_code(RTC::RTC_OK) {};
      void operator()(Port_ptr p)
      {
	if (!port_ref->_is_equivalent(p))
	  {
	    ReturnCode_t retval;
	    retval = p->disconnect(connector_profile.connector_id);
	    if (retval != RTC::RTC_OK)
	      {
		return_code = retval;
	      }
	  }
      }
    };
    
    
    /*!
     * @if jp
     * @brief Port ̑Sڑs Functor
     * @else
     * @brief A functor to disconnect all Ports
     * @endif
     */
    struct disconnect_all_func
    {
      ReturnCode_t return_code;
      PortBase* port;
      
      disconnect_all_func() {};
      disconnect_all_func(PortBase* p) 
	: return_code(RTC::RTC_OK), port(p) {};
      void operator()(ConnectorProfile& p)
      {
	ReturnCode_t retval;
	retval = port->disconnect(p.connector_id);
	if (retval != RTC::RTC_OK)
	  {
	    return_code = retval;
	  }
      }
    };
    

    /*!
     * @if jp
     * @brief name  polarity  interface T Functor
     * @else
     * @brief A functor to find interface from name and polarity
     * @endif
     */
    struct find_interface
    {
      find_interface(const char* name, PortInterfacePolarity pol)
	: m_name(name), m_pol(pol)
      {}

      bool operator()(const PortInterfaceProfile& prof)
      {
	std::string name(CORBA::string_dup(prof.instance_name));
	return ((m_name == name) && (m_pol == prof.polarity));
      }
      std::string m_name;
      PortInterfacePolarity m_pol;
    };
  };
};
#endif // PortBase_h
